# Import gobject and pigment modules and initialize threading support
import sys, gobject, pgm
gobject.threads_init()

# Defines a pigment custom triangle geometry object
class Triangle(pgm.Geometry):

    def __init__(self, width, height):
        # Attributes and indices are stored as lists
        positions = [ -width * 0.5, -height * 0.5, 0.0,
                      -width * 0.5, height * 0.5, 0.0,
                       width * 0.5, -height * 0.5, 0.0 ]
        colors = [ 255, 255, 0, 255, 255, 0, 255, 255, 0, 255, 255, 255 ]
        indices = [ 0, 1, 2 ]

        # Create and set a vertex buffer containing 3 vertices using 3-component
        # positions stored as floats and colors stored as unsigned bytes
        vbuffer = pgm.VertexBuffer(3, pgm.VERTEX_BUFFER_FLOAT, 3,
                                   pgm.VERTEX_BUFFER_UINT8,
                                   pgm.VERTEX_BUFFER_NONE, 0)
        vbuffer.set_positions(0, 3, positions)
        vbuffer.set_colors(0, 3, colors, False)

        # Create a mesh to index the vertex buffer
        mesh = pgm.Mesh(3, pgm.MESH_TRIANGLE_LIST)
        mesh.set_culling(pgm.MESH_CULL_NONE)
        mesh.set_indices(0, 3, indices)

        # Chain up to the geometry constructor giving it the vertex buffer
        pgm.Geometry.__init__(self, vbuffer)

        # Add the mesh and flag the geometry as rendered
        self.add_meshes(mesh)
        self.set_picked(True)
        self.set_rendered(True)

    # Override the Node map virtual method
    def do_map(self, renderer, scale_x, scale_y):
        pgm.Geometry.do_map(self, renderer, scale_x, scale_y)
        print 'map vmethod', renderer, scale_x, scale_y

    # Override the Node unmap virtual method
    def do_unmap(self):
        pgm.Geometry.do_unmap(self)
        print 'unmap vmethod'

    # Override the Node reproject virtual method
    def do_reproject(self, scale_x, scale_y):
        print 'reproject vmethod', scale_x, scale_y

    # Override the Geometry mouse_press virtual method
    def do_mouse_press(self, message, hit):
        print 'mouse_press vmethod'
        pgm.Geometry.do_mouse_press(self, message, hit)

    # Override the Geometry mouse_pressure virtual method
    def do_mouse_pressure(self, message, hit):
        print 'mouse_pressure vmethod'
        pgm.Geometry.do_mouse_pressure(self, message, hit)

    # Override the Geometry mouse_release virtual method
    def do_mouse_release(self, message, hit):
        print 'mouse_release vmethod'
        pgm.Geometry.do_mouse_release(self, message, hit)

    # Override the Geometry mouse_click virtual method
    def do_mouse_click(self, message, hit):
        print 'mouse_click vmethod'
        pgm.Geometry.do_mouse_click(self, message, hit)

    # Override the Geometry mouse_scroll virtual method
    def do_mouse_scroll(self, message, hit):
        print 'mouse_scroll vmethod'
        pgm.Geometry.do_mouse_scroll(self, message, hit)

    # Override the Geometry mouse_enter virtual method
    def do_mouse_enter(self):
        print 'mouse_enter vmethod'
        pgm.Geometry.do_mouse_enter(self)

    # Override the Geometry mouse_leave virtual method
    def do_mouse_leave(self):
        print 'mouse_leave vmethod'
        pgm.Geometry.do_mouse_leave(self)

    # Override the Geometry mouse_hover virtual method
    def do_mouse_hover(self, message, hit):
        print 'mouse_hover vmethod'
        pgm.Geometry.do_mouse_hover(self, message, hit)

# Mandatory step needed to register the new gobject type
gobject.type_register(Triangle)

# Renderer delete message signal handler
def on_delete(renderer, message, mainloop):
    mainloop.quit()

def main(args):
    # Create the OpenGL renderer
    renderer = pgm.RendererFactory.make('opengl')
    renderer.set_window_title('Custom geometry')
    renderer.set_window_size(500, 500)
    renderer.set_mouse_picking(True)

    # Create the scene
    scene = pgm.Scene()
    scene.set_perspective_projection(500.0, 500.0, 11.0, 10.0, -10.0)

    # Create our custom geometry and add it to the scene
    triangle = Triangle(200.0, 200.0)
    triangle.set_visible(True)
    triangle.translate(250.0, 250.0, 0.0)
    scene.add_children(triangle)

    # Create a main loop using the default main context
    mainloop = gobject.MainLoop()

    # Connect renderer signal handlers
    renderer.connect('delete-message', on_delete, mainloop)

    # Associate the scene to the renderer and start the main loop
    renderer.set_scene(scene)
    scene.set_visible(True)
    renderer.set_window_visible(True)
    mainloop.run()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))
